@@ -518,11 +518,11 @@ class HardwareStatsTransformerForm(forms.ModelForm):
 
 
 DataConstantsTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformerConstants, extra=1,
-                                                              form=DataConstantsTransformerForm)
+                                                              form=DataConstantsTransformerForm,can_delete_extra=True)
 HardwareTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareDataTransformer, extra=1,
-                                                         form=HardwareTransformerForm)
+                                                         form=HardwareTransformerForm,can_delete_extra=True)
 HardwareStatsTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareStatsDataTransformer, extra=1,
-                                                              form=HardwareStatsTransformerForm)
+                                                              form=HardwareStatsTransformerForm,can_delete_extra=True)
 
 from django.utils.safestring import mark_safe
 
@@ -530,13 +530,13 @@ from django.utils.safestring import mark_safe
 class DataTransform(forms.ModelForm):
     id = forms.CharField(required=False,
                          widget=forms.HiddenInput(attrs={'class': 'form-control'}))
-    accessory = forms.ModelChoiceField(required=True, queryset=Accessory.objects.all(),
+    accessory = forms.ModelChoiceField(required=False, queryset=Accessory.objects.all(),
                                        widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
-    parent = forms.ModelChoiceField(required=True, queryset=DataTransformer.objects.all(),
+    parent = forms.ModelChoiceField(required=False, queryset=DataTransformer.objects.all(),
                                     widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
-    type = forms.ModelChoiceField(required=True, queryset=DataTransformerTypes.objects.all())
+    type = forms.ModelChoiceField(required=False, queryset=DataTransformerTypes.objects.all())
 
     def __init__(self, *args, **kwargs):
         print('Init Data Transform')
@@ -550,7 +550,8 @@ class DataTransform(forms.ModelForm):
 
                 dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
                                                                      fk_name='parent', form=DataTransform,
-                                                                     extra=self.instance.get_children().count() + 1
+                                                                     extra=self.instance.get_children().count() + 1,
+                                                                     can_delete_extra=True
                                                                      )
                 self.nested_data_transformer_form = dataTransformerInlineFormset(
                     queryset=self.instance.get_children(),
@@ -569,32 +570,34 @@ class DataTransform(forms.ModelForm):
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None
                 )
-            self.nested_constants_form = DataConstantsTransformerInlineFormset(
-                instance=self.instance,
-                data=self.data if self.is_bound else None,
-                files=self.files if self.is_bound else None,
-                initial=[{'data_transformer': self.instance}]
 
-            )
-
-            self.nested_hardware_outputs_form = HardwareTransformerInlineFormset(
-                instance=self.instance,
-                data=self.data if self.is_bound else None,
-                files=self.files if self.is_bound else None,
-                initial=[{'data_transformer': self.instance}]
-
-            )
-
-            self.nested_hardware_stats_form = HardwareStatsTransformerInlineFormset(
-                instance=self.instance,
-                data=self.data if self.is_bound else None,
-                files=self.files if self.is_bound else None,
-                initial=[{'data_transformer': self.instance}]
-
-            )
         except Exception:
             print('excpt')
 
+        self.nested_constants_form = DataConstantsTransformerInlineFormset(
+            instance=self.instance,
+            data=self.data if self.is_bound else None,
+            files=self.files if self.is_bound else None,
+            initial=[{'data_transformer': self.instance}]
+
+        )
+
+        self.nested_hardware_outputs_form = HardwareTransformerInlineFormset(
+            instance=self.instance,
+            data=self.data if self.is_bound else None,
+            files=self.files if self.is_bound else None,
+            initial=[{'data_transformer': self.instance}]
+
+        )
+
+        self.nested_hardware_stats_form = HardwareStatsTransformerInlineFormset(
+            instance=self.instance,
+            data=self.data if self.is_bound else None,
+            files=self.files if self.is_bound else None,
+            initial=[{'data_transformer': self.instance}]
+
+        )
+
     def as_p(self):
         print('wtf')
         print('instance: ' + str(self.instance.id))
@@ -620,17 +623,33 @@ class DataTransform(forms.ModelForm):
         print('node')
         print('save: ' + str(self.instance.id))
         root_node = super(DataTransform, self).save(commit)
+        # if hasattr(self,
+        #            'nested_constants_form') and self.nested_constants_form is not None and self.nested_constants_form.is_valid():
+        #     self.nested_constants_form.save(commit)
+        # if hasattr(self,
+        #            'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None and self.nested_hardware_outputs_form.is_valid():
+        #     self.nested_hardware_outputs_form.save(commit)
+        # if hasattr(self,
+        #            'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None and self.nested_hardware_stats_form.is_valid():
+        #     self.nested_hardware_stats_form.save(commit)
+        # if hasattr(self,
+        #            'nested_data_transformer_form') and self.nested_data_transformer_form is not None and self.nested_data_transformer_form.is_valid():
+        #     self.nested_data_transformer_form.save(commit)
         if hasattr(self,
-                   'nested_constants_form') and self.nested_constants_form is not None and self.nested_constants_form.is_valid():
+                   'nested_constants_form') and self.nested_constants_form is not None:
+            self.nested_constants_form.is_valid()
             self.nested_constants_form.save(commit)
         if hasattr(self,
-                   'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None and self.nested_hardware_outputs_form.is_valid():
+                   'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:
+            self.nested_hardware_outputs_form.is_valid()
             self.nested_hardware_outputs_form.save(commit)
         if hasattr(self,
-                   'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None and self.nested_hardware_stats_form.is_valid():
+                   'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None:
+            self.nested_hardware_stats_form.is_valid()
             self.nested_hardware_stats_form.save(commit)
         if hasattr(self,
-                   'nested_data_transformer_form') and self.nested_data_transformer_form is not None and self.nested_data_transformer_form.is_valid():
+                   'nested_data_transformer_form') and self.nested_data_transformer_form is not None:
+            self.nested_data_transformer_form.is_valid()
             self.nested_data_transformer_form.save(commit)
         root_node.refresh_from_db()
         return root_node
@@ -663,7 +682,8 @@ class DataTransformRoot(forms.ModelForm):
 
                 dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
                                                                      fk_name='parent', form=DataTransform,
-                                                                     extra=self.instance.get_children().count() + 1
+                                                                     extra=self.instance.get_children().count() + 1,
+                                                                     can_delete_extra=True
                                                                      )
                 self.nested_data_transformer_form = dataTransformerInlineFormset(
                     queryset=self.instance.get_children(),
@@ -732,16 +752,20 @@ class DataTransformRoot(forms.ModelForm):
         print('save: ' + str(self.instance.id))
         root_node = super(DataTransformRoot, self).save(commit)
         if hasattr(self,
-                   'nested_constants_form') and self.nested_constants_form is not None and self.nested_constants_form.is_valid():
+                   'nested_constants_form') and self.nested_constants_form is not None:
+            self.nested_constants_form.is_valid()
             self.nested_constants_form.save(commit)
         if hasattr(self,
-                   'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None and self.nested_hardware_outputs_form.is_valid():
+                   'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:
+            self.nested_hardware_outputs_form.is_valid()
             self.nested_hardware_outputs_form.save(commit)
         if hasattr(self,
-                   'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None and self.nested_hardware_stats_form.is_valid():
+                   'nested_hardware_stats_form') and self.nested_hardware_stats_form is not None:
+            self.nested_hardware_stats_form.is_valid()
             self.nested_hardware_stats_form.save(commit)
         if hasattr(self,
-                   'nested_data_transformer_form') and self.nested_data_transformer_form is not None and self.nested_data_transformer_form.is_valid():
+                   'nested_data_transformer_form') and self.nested_data_transformer_form is not None:
+            self.nested_data_transformer_form.is_valid()
             self.nested_data_transformer_form.save(commit)
         root_node.refresh_from_db()
         return root_node.get_root()
