@@ -210,3 +210,14 @@ class HardwareIOSerializer(serializers.ModelSerializer):
     class Meta:
         model = Hardware
         fields = '__all__'
+
+class DataTransformerTreeSerializer(serializers.ModelSerializer):
+    children = serializers.SerializerMethodField(source='get_children')
+    class Meta:
+        model=DataTransformer
+        fields = '__all__'  # add here rest of the fields from model
+
+    def get_children(self, obj):
+        children = self.context['children'].get(obj.id, [])
+        serializer = DataTransformerTreeSerializer(children, many=True, context=self.context)
+        return serializer.data
\ No newline at end of file
